@@ -1,24 +1,23 @@
-# -*- coding: utf-8 -*-
-from __future__ import absolute_import, division, print_function
-from multiprocess import Pool
-import simplejson as json
-import six
 import sys
 
-from cytoolz import compose
+import click
+import h5py
 import numpy as np
 import pandas as pd
-import h5py
-
-from ._util import parse_bins, parse_kv_list_param, parse_field_param
-from . import cli, get_logger
-import click
+import simplejson as json
+from cytoolz import compose
+from multiprocess import Pool
 
 from ..create import (
+    HDF5Aggregator,
+    PairixAggregator,
+    TabixAggregator,
+    aggregate_records,
     create_cooler,
-    sanitize_records, aggregate_records,
-    TabixAggregator, HDF5Aggregator, PairixAggregator,
+    sanitize_records,
 )
+from . import cli, get_logger
+from ._util import parse_bins, parse_field_param, parse_kv_list_param
 
 _pandas_version = pd.__version__.split('.')
 if int(_pandas_version[0]) > 0:
@@ -87,8 +86,9 @@ def cload():
     pass
 
 
+# flake8: noqa
 def register_subcommand(func):
-    return (  # noqa
+    return (
         cload.command()(
             click.argument(
                 "bins",
@@ -147,7 +147,7 @@ def hiclib(bins, pairs_path, cool_path, metadata, assembly, chunksize):
     chromsizes, bins = parse_bins(bins)
 
     if metadata is not None:
-        with open(metadata, 'r') as f:
+        with open(metadata) as f:
             metadata = json.load(f)
 
     with h5py.File(pairs_path, 'r') as h5pairs:
@@ -212,16 +212,14 @@ def tabix(bins, pairs_path, cool_path, metadata, assembly, nproc, zero_based, ma
     chromsizes, bins = parse_bins(bins)
 
     if metadata is not None:
-        with open(metadata, 'r') as f:
+        with open(metadata) as f:
             metadata = json.load(f)
 
     try:
         if nproc > 1:
             pool = Pool(nproc)
-            logger.info("Using {} cores".format(nproc))
+            logger.info(f"Using {nproc} cores")
             map = pool.imap
-        else:
-            map = six.moves.map
 
         opts = {}
         if 'chrom2' in kwargs:
@@ -290,16 +288,14 @@ def pairix(bins, pairs_path, cool_path, metadata, assembly, nproc, zero_based, m
     chromsizes, bins = parse_bins(bins)
 
     if metadata is not None:
-        with open(metadata, 'r') as f:
+        with open(metadata) as f:
             metadata = json.load(f)
 
     try:
         if nproc > 1:
             pool = Pool(nproc)
-            logger.info("Using {} cores".format(nproc))
+            logger.info(f"Using {nproc} cores")
             map = pool.imap
-        else:
-            map = six.moves.map
 
         iterator = PairixAggregator(
             pairs_path,
@@ -430,6 +426,13 @@ def pairix(bins, pairs_path, cool_path, metadata, assembly, nproc, zero_based, m
     "See http://docs.h5py.org/en/stable/high/dataset.html#filter-pipeline "
     "for more details."
 )
+@click.option(
+    "--append", "-a",
+    is_flag=True,
+    default=False,
+    help="Pass this flag to append the output cooler to an existing file "
+         "instead of overwriting the file."
+)
 # @click.option(
 #     "--format", "-f",
 #     help="Preset data format.",
@@ -437,7 +440,8 @@ def pairix(bins, pairs_path, cool_path, metadata, assembly, nproc, zero_based, m
 # --sep
 def pairs(bins, pairs_path, cool_path, metadata, assembly, chunksize,
           zero_based, comment_char, input_copy_status, no_symmetric_upper,
-          field, temp_dir, no_delete_temp, max_merge, storage_options, **kwargs):
+          field, temp_dir, no_delete_temp, max_merge, storage_options,
+          append, **kwargs):
     """
     Bin any text file or stream of pairs.
 
@@ -458,7 +462,7 @@ def pairs(bins, pairs_path, cool_path, metadata, assembly, chunksize,
             tril_action = 'drop'
 
     if metadata is not None:
-        with open(metadata, 'r') as f:
+        with open(metadata) as f:
             metadata = json.load(f)
 
     input_field_names = [
@@ -595,5 +599,6 @@ def pairs(bins, pairs_path, cool_path, metadata, assembly, chunksize,
         ensure_sorted=False,
         symmetric_upper=symmetric_upper,
         h5opts=h5opts,
-        ordered=False
+        ordered=False,
+        mode="a" if append else "w"
     )
