@@ -1,24 +1,20 @@
-# -*- coding: utf-8 -*-
-from __future__ import division, print_function
-from math import ceil
 import shlex
-import sys
 import warnings
+from math import ceil
 
-from ._util import parse_field_param
-from . import cli, get_logger
 import click
 
+from .. import api
+from ..parallel import lock
 from ..reduce import (
-    legacy_zoomify,
-    zoomify_cooler,
-    get_quadtree_depth,
     HIGLASS_TILE_DIM,
-    preferred_sequence
+    legacy_zoomify,
+    preferred_sequence,
+    zoomify_cooler,
 )
 from ..util import parse_cooler_uri
-from ..tools import lock
-from .. import api
+from . import cli, get_logger
+from ._util import parse_field_param
 
 
 def invoke_balance(args, resolutions, outfile):
@@ -29,20 +25,20 @@ def invoke_balance(args, resolutions, outfile):
         args = []
     else:
         args = shlex.split(args)
-    logger.debug("Balancing args: {}".format(args))
+    logger.debug(f"Balancing args: {args}")
 
     for res in resolutions:
         uri = outfile + "::resolutions/" + str(res)
         if "weight" in api.Cooler(uri).bins():
             continue
-        logger.info("Balancing zoom level with bin size {}".format(res))
+        logger.info(f"Balancing zoom level with bin size {res}")
 
         try:
             balance_cmd.main(
                 args=[uri] + args, prog_name='cooler'
             )
         except SystemExit as e:
-            exc_info = sys.exc_info()
+            # exc_info = sys.exc_info()
             exit_code = e.code
 
             if exit_code is None:
@@ -117,9 +113,7 @@ def invoke_balance(args, resolutions, outfile):
     is_flag=True,
     default=False,
 )
-@click.pass_context
 def zoomify(
-    ctx,
     cool_uri,
     nproc,
     chunksize,
@@ -145,8 +139,8 @@ def zoomify(
     else:
         outfile, _ = parse_cooler_uri(out)
 
-    logger.info('Recursively aggregating "{}"'.format(cool_uri))
-    logger.info('Writing to "{}"'.format(outfile))
+    logger.info(f'Recursively aggregating "{cool_uri}"')
+    logger.info(f'Writing to "{outfile}"')
 
     if legacy:
         n_zooms, zoom_levels = legacy_zoomify(
@@ -160,20 +154,20 @@ def zoomify(
                 balance_args = []
             else:
                 balance_args = shlex.split(balance_args)
-            logger.debug("Balancing args: {}".format(balance_args))
+            logger.debug(f"Balancing args: {balance_args}")
 
             for level, res in reversed(list(zoom_levels.items())):
                 uri = outfile + "::" + str(level)
                 if level == str(n_zooms):
                     if "weight" in api.Cooler(uri).bins():
                         continue
-                logger.info("Balancing zoom level {}, bin size {}".format(level, res))
+                logger.info(f"Balancing zoom level {level}, bin size {res}")
                 try:
                     balance_cmd.main(
                         args=[uri] + balance_args, prog_name='cooler'
                     )
                 except SystemExit as e:
-                    exc_info = sys.exc_info()
+                    # exc_info = sys.exc_info()
                     exit_code = e.code
 
                     if exit_code is None:
