@@ -1,45 +1,42 @@
-# -*- coding: utf-8 -*-
-from __future__ import absolute_import, print_function, division
-from datetime import datetime
-from six.moves import map
-from pandas.api.types import is_categorical, is_integer
 import os.path as op
-import pandas as pd
-import numpy as np
 import posixpath
 import tempfile
 import warnings
+from datetime import datetime
+
 import h5py
+import numpy as np
+import pandas as pd
 import simplejson as json
-import six
+from pandas.api.types import is_categorical_dtype
 
-from .._version import __version__, __format_version__, __format_version_scool__
 from .._logging import get_logger
-from ..core import put, get
+from .._version import __format_version__, __format_version_scool__, __version__
+from ..core import get, put
 from ..util import (
-    parse_cooler_uri,
-    get_chromsizes,
     get_binsize,
-    infer_meta,
+    get_chromsizes,
     get_meta,
+    infer_meta,
+    parse_cooler_uri,
     rlencode,
 )
-from ._ingest import validate_pixels
 from . import (
-    MAGIC,
-    MAGIC_SCOOL,
-    URL,
+    BIN1OFFSET_DTYPE,
+    BIN_DTYPE,
     CHROM_DTYPE,
     CHROMID_DTYPE,
+    CHROMOFFSET_DTYPE,
     CHROMSIZE_DTYPE,
     COORD_DTYPE,
-    BIN_DTYPE,
     COUNT_DTYPE,
-    CHROMOFFSET_DTYPE,
-    BIN1OFFSET_DTYPE,
-    PIXEL_FIELDS,
+    MAGIC,
+    MAGIC_SCOOL,
     PIXEL_DTYPES,
+    PIXEL_FIELDS,
+    URL,
 )
+from ._ingest import validate_pixels
 
 logger = get_logger("cooler.create")
 
@@ -126,7 +123,7 @@ def write_bins(grp, bins, chromnames, h5opts, chrom_as_enum=True):
         else:
             raise
     if not chrom_as_enum:
-        chrom_dset.attrs["enum_path"] = u"/chroms/name"
+        chrom_dset.attrs["enum_path"] = "/chroms/name"
 
     grp.create_dataset(
         "start", shape=(n_bins,), dtype=COORD_DTYPE, data=bins["start"], **h5opts
@@ -214,13 +211,13 @@ def write_pixels(filepath, grouppath, columns, iterable, h5opts, lock):
     for i, chunk in enumerate(iterable):
 
         if isinstance(chunk, pd.DataFrame):
-            chunk = {k: v.values for k, v in six.iteritems(chunk)}
+            chunk = {k: v.values for k, v in chunk.items()}
 
         try:
             if lock is not None:
                 lock.acquire()
 
-            logger.debug("writing chunk {}".format(i))
+            logger.debug(f"writing chunk {i}")
 
             with h5py.File(filepath, "r+") as fw:
                 grp = fw[grouppath]
@@ -247,7 +244,7 @@ def index_pixels(grp, n_bins, nnz):
     bin1 = grp["bin1_id"]
     bin1_offset = np.zeros(n_bins + 1, dtype=BIN1OFFSET_DTYPE)
     curr_val = 0
-    for start, length, value in zip(*rlencode(bin1, 1000000)):
+    for start, _length, value in zip(*rlencode(bin1, 1000000)):
         bin1_offset[curr_val : value + 1] = start
         curr_val = value + 1
     bin1_offset[curr_val:] = nnz
@@ -258,7 +255,7 @@ def index_bins(grp, n_chroms, n_bins):
     chrom_ids = grp["chrom"]
     chrom_offset = np.zeros(n_chroms + 1, dtype=CHROMOFFSET_DTYPE)
     curr_val = 0
-    for start, length, value in zip(*rlencode(chrom_ids)):
+    for start, _length, value in zip(*rlencode(chrom_ids)):
         chrom_offset[curr_val : value + 1] = start
         curr_val = value + 1
     chrom_offset[curr_val:] = n_bins
@@ -323,14 +320,13 @@ def write_info(grp, info, scool=False):
     info.setdefault("genome-assembly", "unknown")
     info["metadata"] = json.dumps(info.get("metadata", {}))
     info["creation-date"] = datetime.now().isoformat()
-    info["generated-by"] = six.text_type("cooler-" + __version__)
+    info["generated-by"] = "cooler-" + __version__
     if scool:
         info["format"] = MAGIC_SCOOL
-        info["format-version"] = six.text_type(__format_version_scool__)
-
+        info["format-version"] = __format_version_scool__
     else:
         info["format"] = MAGIC
-        info["format-version"] = six.text_type(__format_version__)
+        info["format-version"] = __format_version__
     info["format-url"] = URL
     grp.attrs.update(info)
 
@@ -342,15 +338,17 @@ def _rename_chroms(grp, rename_dict, h5opts):
         chroms.rename(rename_dict).index.values, dtype=CHROM_DTYPE
     )  # auto-adjusts char length
 
+    # Replace chroms/name
     del grp["chroms/name"]
     grp["chroms"].create_dataset(
         "name", shape=(n_chroms,), dtype=new_names.dtype, data=new_names, **h5opts
     )
 
+    # Replace the bins/chroms enum mapping if applicable
     bins = get(grp["bins"])
     n_bins = len(bins)
-    idmap = dict(zip(new_names, range(n_chroms)))
-    if is_categorical(bins["chrom"]) or is_integer(bins["chrom"]):
+    if is_categorical_dtype(bins["chrom"]):
+        idmap = dict(zip(new_names, range(n_chroms)))
         chrom_ids = bins["chrom"].cat.codes
         chrom_dtype = h5py.special_dtype(enum=(CHROMID_DTYPE, idmap))
         del grp["bins/chrom"]
@@ -421,7 +419,7 @@ def _set_h5opts(h5opts):
     }
     for key in result.keys():
         if key not in available_opts:
-            raise ValueError("Unknown storage option '{}'.".format(key))
+            raise ValueError(f"Unknown storage option '{key}'.")
     result.setdefault("compression", "gzip")
     if result["compression"] == "gzip" and "compression_opts" not in result:
         result["compression_opts"] = 6
@@ -479,7 +477,7 @@ def create(
             "Note that the `chromsizes` argument is now deprecated: "
             "see documentation for `create`."
         )
-    if append_scool == True and scool_root_uri is None:
+    if append_scool and scool_root_uri is None:
         raise ValueError(
             "If the parameter `append_scool` is set, the parameter `scool_root_uri` must be defined."
         )
@@ -487,7 +485,7 @@ def create(
 
     for col in ["chrom", "start", "end"]:
         if col not in bins.columns:
-            raise ValueError("Missing column from bin table: '{}'.".format(col))
+            raise ValueError(f"Missing column from bin table: '{col}'.")
 
     # Populate expected pixel column names. Include user-provided value
     # columns.
@@ -518,7 +516,7 @@ def create(
         dask_df = ()
 
     if isinstance(pixels, dask_df):
-        iterable = map(lambda x: x.compute(), pixels.to_delayed())
+        iterable = (x.compute() for x in pixels.to_delayed())
         input_columns = infer_meta(pixels).columns
     elif isinstance(pixels, pd.DataFrame):
         iterable = (pixels,)
@@ -536,7 +534,7 @@ def create(
             if col not in input_columns:
                 col_type = "Standard" if col in PIXEL_FIELDS else "User"
                 raise ValueError(
-                    "{} column not found in input: '{}'".format(col_type, col)
+                    f"{col_type} column not found in input: '{col}'"
                 )
 
     # Prepare chroms and bins
@@ -544,7 +542,7 @@ def create(
     bins["chrom"] = bins["chrom"].astype(object)
     chromsizes = get_chromsizes(bins)
     try:
-        chromsizes = six.iteritems(chromsizes)
+        chromsizes = chromsizes.items()
     except AttributeError:
         pass
     chromnames, lengths = zip(*chromsizes)
@@ -571,7 +569,7 @@ def create(
 
     # Create root group
     with h5py.File(file_path, mode) as f:
-        logger.info('Creating cooler at "{}::{}"'.format(file_path, group_path))
+        logger.info(f'Creating cooler at "{file_path}::{group_path}"')
         if group_path == "/":
             for name in ["chroms", "bins", "pixels", "indexes"]:
                 if name in f:
@@ -594,8 +592,8 @@ def create(
 
             # hard link to root bins table, but only the three main datasets
             dst[dst_group]["bins/chrom"] = src["bins/chrom"]
-            dst[dst_group]["bins/start"]= src["bins/start"]
-            dst[dst_group]["bins/end"]= src["bins/end"]
+            dst[dst_group]["bins/start"] = src["bins/start"]
+            dst[dst_group]["bins/end"] = src["bins/end"]
 
             # create per cell the additional columns e.g. 'weight'
             # these columns are individual for each cell
@@ -657,9 +655,9 @@ def create(
 
         logger.info("Writing info")
         info = {}
-        info["bin-type"] = u"fixed" if binsize is not None else u"variable"
-        info["bin-size"] = binsize if binsize is not None else u"null"
-        info["storage-mode"] = u"symmetric-upper" if symmetric_upper else u"square"
+        info["bin-type"] = "fixed" if binsize is not None else "variable"
+        info["bin-size"] = binsize if binsize is not None else "null"
+        info["storage-mode"] = "symmetric-upper" if symmetric_upper else "square"
         info["nchroms"] = n_chroms
         info["nbins"] = n_bins
         info["sum"] = ncontacts
@@ -678,7 +676,7 @@ def create_from_unordered(
     columns=None,
     dtypes=None,
     mode=None,
-    mergebuf=int(20e6),
+    mergebuf=20_000_000,
     delete_temp=True,
     temp_dir=None,
     max_merge=200,
@@ -720,7 +718,7 @@ def create_from_unordered(
     for i, chunk in enumerate(chunks):
         uri = tf.name + "::" + str(i)
         uris.append(uri)
-        logger.info("Writing chunk {}: {}".format(i, uri))
+        logger.info(f"Writing chunk {i}: {uri}")
         create(uri, bins, chunk, columns=columns, dtypes=dtypes, mode="a", **kwargs)
 
     # Merge passes
@@ -739,9 +737,9 @@ def create_from_unordered(
             chunk_subset = CoolerMerger(
                 [Cooler(uri) for uri in uris[lo:hi]], mergebuf, columns=columns
             )
-            uri = tf2.name + "::" + "{}-{}".format(lo, hi)
+            uri = tf2.name + "::" + f"{lo}-{hi}"
             uris2.append(uri)
-            logger.info("Merging chunks {}-{}: {}".format(lo, hi, uri))
+            logger.info(f"Merging chunks {lo}-{hi}: {uri}")
             create(
                 uri,
                 bins,
@@ -761,7 +759,7 @@ def create_from_unordered(
     chunks = CoolerMerger(
         [Cooler(uri) for uri in final_uris], mergebuf, columns=columns
     )
-    logger.info("Merging into {}".format(cool_uri))
+    logger.info(f"Merging into {cool_uri}")
     create(cool_uri, bins, chunks, columns=columns, dtypes=dtypes, mode=mode, **kwargs)
 
     del temp_files
@@ -798,7 +796,8 @@ def append(cool_uri, table, data, chunked=False, force=False, h5opts=None, lock=
     file_path, group_path = parse_cooler_uri(cool_uri)
 
     try:
-        from dask.dataframe import DataFrame as dask_df, Series as dask_series
+        from dask.dataframe import DataFrame as dask_df
+        from dask.dataframe import Series as dask_series
     except (ImportError, AttributeError):
         dask_df = ()
         dask_series = ()
@@ -817,7 +816,7 @@ def append(cool_uri, table, data, chunked=False, force=False, h5opts=None, lock=
             if name in h5[table]:
                 if not force:
                     raise ValueError(
-                        "'{}' column already exists. ".format(name)
+                        f"'{name}' column already exists. "
                         + "Use --force option to overwrite."
                     )
                 else:
@@ -963,7 +962,7 @@ def create_cooler(
     ordered=False,
     symmetric_upper=True,
     mode="w",
-    mergebuf=int(20e6),
+    mergebuf=20_000_000,
     delete_temp=True,
     temp_dir=None,
     max_merge=200,
@@ -1071,7 +1070,7 @@ def create_scool(
     ordered=False,
     symmetric_upper=True,
     mode="w",
-    mergebuf=int(20e6),
+    mergebuf=20_000_000,
     delete_temp=True,
     temp_dir=None,
     max_merge=200,
@@ -1081,7 +1080,8 @@ def create_scool(
     ensure_sorted=False,
     h5opts=None,
     lock=None,
-    **kwargs):
+    **kwargs
+):
     r"""
     Create a single-cell (scool) file.
 
@@ -1150,7 +1150,7 @@ def create_scool(
 
     for col in ["chrom", "start", "end"]:
         if col not in bins.columns:
-            raise ValueError("Missing column from bin table: '{}'.".format(col))
+            raise ValueError(f"Missing column from bin table: '{col}'.")
 
     # Populate dtypes for expected pixel columns, and apply user overrides.
     if dtypes is None:
@@ -1161,17 +1161,17 @@ def create_scool(
         dtypes.update(dtypes_)
 
     # Determine the appropriate iterable
-    try:
-        from dask.dataframe import DataFrame as dask_df
-    except (ImportError, AttributeError):  # pragma: no cover
-        dask_df = ()
+    # try:
+    #     from dask.dataframe import DataFrame as dask_df
+    # except (ImportError, AttributeError):  # pragma: no cover
+    #     dask_df = ()
 
     # Prepare chroms and bins
     bins = bins.copy()
     bins["chrom"] = bins["chrom"].astype(object)
     chromsizes = get_chromsizes(bins)
     try:
-        chromsizes = six.iteritems(chromsizes)
+        chromsizes = chromsizes.items()
     except AttributeError:
         pass
     chromnames, lengths = zip(*chromsizes)
@@ -1184,7 +1184,7 @@ def create_scool(
 
     # Create root group
     with h5py.File(file_path, mode) as f:
-        logger.info('Creating cooler at "{}::{}"'.format(file_path, group_path))
+        logger.info(f'Creating cooler at "{file_path}::{group_path}"')
         if group_path == "/":
             for name in ["chroms", "bins"]:
                 if name in f:
@@ -1212,8 +1212,8 @@ def create_scool(
 
         logger.info("Writing info")
         info = {}
-        info["bin-type"] = u"fixed" if binsize is not None else u"variable"
-        info["bin-size"] = binsize if binsize is not None else u"null"
+        info["bin-type"] = "fixed" if binsize is not None else "variable"
+        info["bin-size"] = binsize if binsize is not None else "null"
         info["nchroms"] = n_chroms
         info["ncells"] = len(cell_name_pixels_dict)
         info["nbins"] = n_bins
