@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 """
 Contact Binners
 ~~~~~~~~~~~~~~~
@@ -7,29 +6,26 @@ Binners are iterators that convert input data of various flavors into a
 properly sorted, chunked stream of binned contacts.
 
 """
-from __future__ import absolute_import, division, print_function
-from collections import OrderedDict, Counter
-from bisect import bisect_left
-from functools import partial
 import itertools
 import warnings
-import six
+from bisect import bisect_left
+from collections import Counter, OrderedDict
+from functools import partial
 
-from pandas.api.types import is_integer_dtype
 import numpy as np
 import pandas as pd
+from pandas.api.types import is_integer_dtype
 
 from .._logging import get_logger
 from ..util import (
-    rlencode,
-    partition,
-    check_bins,
-    get_chromsizes,
     GenomeSegmentation,
     balanced_partition,
+    check_bins,
+    get_chromsizes,
+    partition,
+    rlencode,
 )
 
-
 logger = get_logger("cooler.create")
 
 
@@ -38,28 +34,28 @@ class BadInputError(ValueError):
 
 
 SANITIZE_PRESETS = {
-    "bg2": dict(
-        decode_chroms=True,
-        is_one_based=False,
-        tril_action="reflect",
-        chrom_field="chrom",
-        anchor_field="start",
-        sided_fields=("chrom", "start", "end"),
-        suffixes=("1", "2"),
-        sort=True,
-        validate=True,
-    ),
-    "pairs": dict(
-        decode_chroms=True,
-        is_one_based=False,
-        tril_action="reflect",
-        chrom_field="chrom",
-        anchor_field="pos",
-        sided_fields=("chrom", "pos"),
-        suffixes=("1", "2"),
-        sort=False,
-        validate=True,
-    ),
+    "bg2": {
+        "decode_chroms": True,
+        "is_one_based": False,
+        "tril_action": "reflect",
+        "chrom_field": "chrom",
+        "anchor_field": "start",
+        "sided_fields": ("chrom", "start", "end"),
+        "suffixes": ("1", "2"),
+        "sort": True,
+        "validate": True,
+    },
+    "pairs": {
+        "decode_chroms": True,
+        "is_one_based": False,
+        "tril_action": "reflect",
+        "chrom_field": "chrom",
+        "anchor_field": "pos",
+        "sided_fields": ("chrom", "pos"),
+        "suffixes": ("1", "2"),
+        "sort": False,
+        "validate": True,
+    },
 }
 
 
@@ -92,7 +88,7 @@ def _sanitize_records(
             for col, dt in [("chrom1", chrom1_ids.dtype), ("chrom2", chrom2_ids.dtype)]:
                 if not is_integer_dtype(dt):
                     raise BadInputError(
-                        "`{}` column is non-integer. ".format(col)
+                        f"`{col}` column is non-integer. "
                         + "If string, use `decode_chroms=True` to convert to enum"
                     )
 
@@ -183,7 +179,7 @@ def _sanitize_records(
                     )
                 )
             else:
-                raise ValueError("Unknown tril_action value: '{}'".format(tril_action))
+                raise ValueError(f"Unknown tril_action value: '{tril_action}'")
 
     # Assign bin IDs from bin table
     chrom_binoffset = gs.chrom_binoffset
@@ -284,11 +280,12 @@ def sanitize_records(bins, schema=None, **kwargs):
     """
     if schema is not None:
         try:
-            options = SANITIZE_PRESETS[schema].copy()
+            options = SANITIZE_PRESETS[schema]
         except KeyError:
-            raise ValueError("Unknown schema: '{}'".format(schema))
+            raise ValueError(f"Unknown schema: '{schema}'") from None
     else:
         options = {}
+    options = options.copy()
     options.update(**kwargs)
     chromsizes = get_chromsizes(bins)
     options["gs"] = GenomeSegmentation(chromsizes, bins)
@@ -336,7 +333,7 @@ def _sanitize_pixels(
             elif tril_action == "raise":
                 raise BadInputError("Found bin1_id greater than bin2_id")
             else:
-                raise ValueError("Unknown tril_action value: '{}'".format(tril_action))
+                raise ValueError(f"Unknown tril_action value: '{tril_action}'")
 
     return chunk.sort_values([bin1_field, bin2_field]) if sort else chunk
 
@@ -485,7 +482,7 @@ def aggregate_records(sort=True, count=True, agg=None, rename=None):
     return _aggregate_records
 
 
-class ContactBinner(object):
+class ContactBinner:
     """
     Base class for iterable contact binners.
 
@@ -567,7 +564,7 @@ class HDF5Aggregator(ContactBinner):
             if lo == hi:
                 hi = chrom_hi
 
-            logger.info("{} {}".format(lo, hi))
+            logger.info(f"{lo} {hi}")
 
             # load chunk and assign bin IDs to each read side
             table = self._load_chunk(lo, hi)
@@ -611,7 +608,7 @@ class HDF5Aggregator(ContactBinner):
     def __iter__(self):
         for chrom in self.gs.contigs:
             for df in self.aggregate(chrom):
-                yield {k: v.values for k, v in six.iteritems(df)}
+                yield {k: v.values for k, v in df.items()}
 
 
 class TabixAggregator(ContactBinner):
@@ -634,10 +631,11 @@ class TabixAggregator(ContactBinner):
         try:
             import pysam
         except ImportError:
-            raise ImportError("pysam is required to read tabix files")
+            raise ImportError("pysam is required to read tabix files") from None
+
+        import pickle
 
         import dill
-        import pickle
 
         dill.settings["protocol"] = pickle.HIGHEST_PROTOCOL
 
@@ -665,7 +663,7 @@ class TabixAggregator(ContactBinner):
         for chrom in self.gs.contigs:
             if chrom not in self.file_contigs:
                 warnings.warn(
-                    "Did not find contig " + " '{}' in contact list file.".format(chrom)
+                    "Did not find contig " + f" '{chrom}' in contact list file."
                 )
 
         warnings.warn(
@@ -689,7 +687,7 @@ class TabixAggregator(ContactBinner):
         C2 = self.C2
         P2 = self.P2
 
-        logger.info("Binning {}:{}-{}|*".format(chrom1, start, end))
+        logger.info(f"Binning {chrom1}:{start}-{end}|*")
 
         these_bins = self.gs.fetch((chrom1, start, end))
         rows = []
@@ -741,7 +739,7 @@ class TabixAggregator(ContactBinner):
 
                 accumulator.clear()
 
-        logger.info("Finished {}:{}-{}|*".format(chrom1, start, end))
+        logger.info(f"Finished {chrom1}:{start}-{end}|*")
 
         return pd.concat(rows, axis=0) if len(rows) else None
 
@@ -749,7 +747,7 @@ class TabixAggregator(ContactBinner):
         granges = balanced_partition(self.gs, self.n_chunks, self.file_contigs)
         for df in self._map(self.aggregate, granges):
             if df is not None:
-                yield {k: v.values for k, v in six.iteritems(df)}
+                yield {k: v.values for k, v in df.items()}
 
 
 class PairixAggregator(ContactBinner):
@@ -772,10 +770,13 @@ class PairixAggregator(ContactBinner):
         try:
             import pypairix
         except ImportError:
-            raise ImportError("pypairix is required to read pairix-indexed files")
+            raise ImportError(
+                "pypairix is required to read pairix-indexed files"
+            ) from None
+
+        import pickle
 
         import dill
-        import pickle
 
         dill.settings["protocol"] = pickle.HIGHEST_PROTOCOL
 
@@ -828,7 +829,7 @@ class PairixAggregator(ContactBinner):
         for chrom in self.gs.contigs:
             if chrom not in self.file_contigs:
                 warnings.warn(
-                    "Did not find contig " + " '{}' in contact list file.".format(chrom)
+                    "Did not find contig " + f" '{chrom}' in contact list file."
                 )
 
     def aggregate(self, grange):  # pragma: no cover
@@ -847,7 +848,7 @@ class PairixAggregator(ContactBinner):
         P1 = self.P1
         P2 = self.P2
 
-        logger.info("Binning {}:{}-{}|*".format(chrom1, start, end))
+        logger.info(f"Binning {chrom1}:{start}-{end}|*")
 
         f = pypairix.open(filepath, "r")
         these_bins = self.gs.fetch((chrom1, start, end))
@@ -858,7 +859,7 @@ class PairixAggregator(ContactBinner):
         rows = []
         for bin1_id, bin1 in these_bins.iterrows():
 
-            for chrom2, cid2 in six.iteritems(remaining_chroms):
+            for chrom2, cid2 in remaining_chroms.items():
 
                 chrom2_size = chromsizes[chrom2]
 
@@ -910,7 +911,7 @@ class PairixAggregator(ContactBinner):
 
             accumulator.clear()
 
-        logger.info("Finished {}:{}-{}|*".format(chrom1, start, end))
+        logger.info(f"Finished {chrom1}:{start}-{end}|*")
 
         return pd.concat(rows, axis=0) if len(rows) else None
 
@@ -918,7 +919,7 @@ class PairixAggregator(ContactBinner):
         granges = balanced_partition(self.gs, self.n_chunks, self.file_contigs)
         for df in self._map(self.aggregate, granges):
             if df is not None:
-                yield {k: v.values for k, v in six.iteritems(df)}
+                yield {k: v.values for k, v in df.items()}
 
 
 class SparseBlockLoader(ContactBinner):  # pragma: no cover
@@ -937,7 +938,7 @@ class SparseBlockLoader(ContactBinner):  # pragma: no cover
         chrom_ids = bins["chrom"].cat.codes
         self.offsets = np.zeros(n_chroms + 1, dtype=int)
         curr_val = 0
-        for start, length, value in zip(*rlencode(chrom_ids)):
+        for start, _length, value in zip(*rlencode(chrom_ids)):
             self.offsets[curr_val : value + 1] = start
             curr_val = value + 1
         self.offsets[curr_val:] = n_bins
@@ -951,7 +952,7 @@ class SparseBlockLoader(ContactBinner):  # pragma: no cover
             try:
                 block = self.mapping[chrom2, chrom1].T
             except KeyError:
-                warnings.warn("Block for {{{}, {}}} not found".format(chrom1, chrom2))
+                warnings.warn(f"Block for {{{chrom1}, {chrom2}}} not found")
                 raise
         return block
 
@@ -1040,7 +1041,7 @@ class ArrayBlockLoader(ContactBinner):  # pragma: no cover
         chrom_ids = bins["chrom"].cat.codes
         self.offsets = np.zeros(n_chroms + 1, dtype=int)
         curr_val = 0
-        for start, length, value in zip(*rlencode(chrom_ids)):
+        for start, _length, value in zip(*rlencode(chrom_ids)):
             self.offsets[curr_val : value + 1] = start
             curr_val = value + 1
         self.offsets[curr_val:] = n_bins
@@ -1054,7 +1055,7 @@ class ArrayBlockLoader(ContactBinner):  # pragma: no cover
             try:
                 block = self.mapping[chrom2, chrom1].T
             except KeyError:
-                warnings.warn("Block for {{{}, {}}} not found".format(chrom1, chrom2))
+                warnings.warn(f"Block for {{{chrom1}, {chrom2}}} not found")
                 raise
         return block
 
