@@ -1,25 +1,22 @@
-# -*- coding: utf-8 -*-
-from __future__ import absolute_import, print_function, division
-from datetime import datetime
+import os
+
 # from textwrap import dedent
 import warnings
+from datetime import datetime
+
 import simplejson as json
-import os
-from six import PY2
-import six
 
 try:
     from simplejson import JSONDecodeError
 except ImportError:
     JSONDecodeError = ValueError  # PY35+
 
+import h5py
 from asciitree import BoxStyle, LeftAligned
 from asciitree.traversal import Traversal
-import numpy as np
-import h5py
 
-from .util import parse_cooler_uri, natsorted
-from .create import MAGIC, URL, MAGIC_SCOOL
+from .create import MAGIC, MAGIC_SCOOL
+from .util import natsorted, parse_cooler_uri
 
 __all__ = ["is_cooler", "is_multires_file", "list_coolers", "cp", "mv", "ln"]
 
@@ -42,7 +39,7 @@ def decode_attr_value(obj):
         o = obj.item()
     elif hasattr(obj, "tolist"):
         o = obj.tolist()
-    elif isinstance(obj, six.string_types):
+    elif isinstance(obj, str):
         try:
             o = datetime.strptime(obj, "%Y-%m-%dT%H:%M:%S.%f")
         except ValueError:
@@ -55,7 +52,7 @@ def decode_attr_value(obj):
     return o
 
 
-class TreeNode(object):
+class TreeNode:
     def __init__(self, obj, depth=0, level=None):
         self.obj = obj
         self.depth = depth
@@ -77,7 +74,7 @@ class TreeNode(object):
     def get_text(self):
         name = self.obj.name.split("/")[-1] or "/"
         if hasattr(self.obj, "shape"):
-            name += " {} {}".format(self.obj.shape, self.obj.dtype)
+            name += f" {self.obj.shape} {self.obj.dtype}"
         return name
 
 
@@ -107,7 +104,7 @@ def _is_cooler(grp):
     if fmt == MAGIC:
         keys = ("chroms", "bins", "pixels", "indexes")
         if not all(name in grp.keys() for name in keys):
-            warnings.warn("Cooler path {} appears to be corrupt".format(grp.name))
+            warnings.warn(f"Cooler path {grp.name} appears to be corrupt")
         return True
     return False
 
@@ -116,7 +113,7 @@ def is_cooler(uri):
     """
     Determine if a URI string references a cooler data collection.
     Returns False if the file or group path doesn't exist.
-    
+
     """
     filepath, grouppath = parse_cooler_uri(uri)
     if not h5py.is_hdf5(filepath):
@@ -153,7 +150,7 @@ def is_scool_file(filepath):
 
     """
     if not h5py.is_hdf5(filepath):
-        raise OSError("'{}' is not an HDF5 file.".format(filepath))
+        raise OSError(f"'{filepath}' is not an HDF5 file.")
         return False
 
     with h5py.File(filepath) as f:
@@ -161,7 +158,7 @@ def is_scool_file(filepath):
         if fmt == MAGIC_SCOOL:
             keys = ("chroms", "bins", "cells")
             if not all(name in f.keys() for name in keys):
-                warnings.warn("Scooler path {} appears to be corrupt".format(grp.name))
+                warnings.warn("Scool file appears to be corrupt")
                 return False
             if "cells" in f.keys() and len(f["cells"].keys()) > 0:
                 for cells in f["cells"].keys():
@@ -186,7 +183,7 @@ def list_coolers(filepath):
 
     """
     if not h5py.is_hdf5(filepath):
-        raise OSError("'{}' is not an HDF5 file.".format(filepath))
+        raise OSError(f"'{filepath}' is not an HDF5 file.")
 
     listing = []
 
@@ -215,12 +212,12 @@ def list_scool_cells(filepath):
         Cooler group paths of all cells in the file.
 
     """
-    if is_scool_file(filepath):
+    def _check_cooler(pth, grp):
+        if _is_cooler(grp):
+            listing.append("/" + pth if not pth.startswith("/") else pth)
 
+    if is_scool_file(filepath):
         listing = []
-        def _check_cooler(pth, grp):
-            if _is_cooler(grp):
-                listing.append("/" + pth if not pth.startswith("/") else pth)
         with h5py.File(filepath, "r") as f:
             _check_cooler("/", f)
             visititems(f, _check_cooler)
@@ -228,7 +225,7 @@ def list_scool_cells(filepath):
             listing.remove('/')
         return natsorted(listing)
     else:
-        raise OSError("'{}' is not a scool file.".format(filepath))
+        raise OSError(f"'{filepath}' is not a scool file.")
     return False
 
 
@@ -248,7 +245,7 @@ def ls(uri):
     """
     filepath, grouppath = parse_cooler_uri(uri)
     if not h5py.is_hdf5(filepath):
-        raise OSError("'{}' is not an HDF5 file.".format(filepath))
+        raise OSError(f"'{filepath}' is not an HDF5 file.")
 
     listing = []
 
@@ -270,13 +267,17 @@ def _copy(src_uri, dst_uri, overwrite, link, rename, soft_link):
         raise ValueError('Must provide at most one of: "link", "rename", "soft_link"')
 
     if not os.path.isfile(dst_path) or overwrite:
-        write_mode = "w"
+        dst_write_mode = "w"
     else:
-        write_mode = "r+"
+        dst_write_mode = "r+"
 
-    with h5py.File(src_path, "r+") as src, h5py.File(
-        dst_path, write_mode
-    ) as dst:  # noqa
+    if src_path == dst_path:
+        src_write_mode = "r+"
+    else:
+        src_write_mode = "r"
+
+    with h5py.File(src_path, src_write_mode) as src, \
+         h5py.File(dst_path, dst_write_mode) as dst:  # noqa
 
         if src_path == dst_path:
             if link or rename:
@@ -358,7 +359,7 @@ def tree_widget(group, expand, level):
             "to get the required ipytree dependency for displaying the tree "
             "widget. If using jupyterlab, you also need to run "
             "`jupyter labextension install ipytree`".format(error)
-        )
+        ) from None
 
     result = ipytree.Tree()
     root = TreeNode(group, level=level)
@@ -378,7 +379,7 @@ class TreeTraversal(Traversal):
         return node.get_text()
 
 
-class TreeViewer(object):
+class TreeViewer:
     """
     Generates ascii- or html-based reprs for "Groupy" objects.
     Borrowed with minor modifications from the zarr project
@@ -395,21 +396,21 @@ class TreeViewer(object):
         self.expand = expand
         self.level = level
 
-        self.text_kwargs = dict(horiz_len=2, label_space=1, indent=1)
+        self.text_kwargs = {"horiz_len": 2, "label_space": 1, "indent": 1}
 
-        self.bytes_kwargs = dict(
-            UP_AND_RIGHT="+",
-            HORIZONTAL="-",
-            VERTICAL="|",
-            VERTICAL_AND_RIGHT="+"
-        )
+        self.bytes_kwargs = {
+            "UP_AND_RIGHT": "+",
+            "HORIZONTAL": "-",
+            "VERTICAL": "|",
+            "VERTICAL_AND_RIGHT": "+"
+        }
 
-        self.unicode_kwargs = dict(
-            UP_AND_RIGHT=u"\u2514",
-            HORIZONTAL=u"\u2500",
-            VERTICAL=u"\u2502",
-            VERTICAL_AND_RIGHT=u"\u251C",
-        )
+        self.unicode_kwargs = {
+            "UP_AND_RIGHT": "\u2514",
+            "HORIZONTAL": "\u2500",
+            "VERTICAL": "\u2502",
+            "VERTICAL_AND_RIGHT": "\u251C",
+        }
 
         self.node_cls = node_cls
 
@@ -423,8 +424,7 @@ class TreeViewer(object):
 
         # Unicode characters slip in on Python 3.
         # So we need to straighten that out first.
-        if not PY2:
-            result = result.encode()
+        result = result.encode()
 
         return result
 
@@ -437,14 +437,11 @@ class TreeViewer(object):
         return drawer(root)
 
     def __repr__(self):
-        if PY2:  # pragma: py3 no cover
-            return self.__bytes__()
-        else:  # pragma: py2 no cover
-            return self.__unicode__()
+        return self.__unicode__()
 
-    def _ipython_display_(self):
+    def _repr_mimebundle_(self):
         tree = tree_widget(self.group, expand=self.expand, level=self.level)
-        tree._ipython_display_()
+        tree._repr_mimebundle_()
         return tree
 
 
@@ -462,8 +459,9 @@ def read_attr_tree(group, level):
 
 
 def pprint_attr_tree(uri, level):
+    from io import StringIO
+
     import yaml
-    from six import StringIO
 
     path, group = parse_cooler_uri(uri)
     with h5py.File(path, "r") as f:
