@@ -1,10 +1,11 @@
 import contextlib
-import tempfile
-import shutil
 import os
+import shutil
+import tempfile
 
-from pandas.api.types import is_numeric_dtype
 import numpy as np
+from pandas.api.types import is_numeric_dtype
+
 import cooler
 
 
@@ -22,7 +23,7 @@ def isolated_filesystem():
         os.chdir(cwd)
         try:
             shutil.rmtree(t)
-        except (OSError, IOError):
+        except OSError:
             pass
 
 
