@@ -1,19 +1,16 @@
-# -*- coding: utf-8 -*-
-from __future__ import division, print_function
-from six import iteritems
-from io import BytesIO
-import tempfile
-import os.path as op
 import os
+import os.path as op
+import tempfile
+from io import BytesIO
+
+import h5py
 import numpy as np
 import pandas as pd
-import h5py
 import pytest
-
 from _common import isolated_filesystem
-import cooler.create
-import cooler
 
+import cooler
+import cooler.create
 
 tmp = tempfile.gettempdir()
 testdir = op.dirname(op.realpath(__file__))
@@ -36,13 +33,13 @@ def test_create_append(fp):
     cooler.create.create(
         op.join(tmp, "test.dict.2000kb.cool"),
         bins,
-        {k: v for k, v in iteritems(pixels)},
+        {k: v for k, v in pixels.items()},
     )
     cooler.create.create(op.join(tmp, "test.iter_df.2000kb.cool"), bins, [pixels])
     cooler.create.create(
         op.join(tmp, "test.iter_dict.2000kb.cool"),
         bins,
-        [{k: v for k, v in iteritems(pixels)}],
+        [{k: v for k, v in pixels.items()}],
     )
     ddf = dd.from_pandas(pixels, npartitions=3)
     cooler.create.create(op.join(tmp, "test.ddf.2000kb.cool"), bins, ddf)
@@ -259,7 +256,7 @@ def test_write():
 
 def test_many_contigs():
     chroms = pd.DataFrame({
-        'name': ['scaffold_{:05}'.format(i) for i in range(4000)],
+        'name': [f'scaffold_{i:05}' for i in range(4000)],
         'length': np.full(4000, 20),
     }, columns=['name', 'length'])
     bins = cooler.util.binnify(chroms.set_index('name')['length'], 10)
@@ -283,7 +280,7 @@ def test_many_contigs():
     cooler.create._create._rename_chroms(
         f,
         {
-            'scaffold_{:05}'.format(i) : 'contig_{:05}'.format(i)
+            f'scaffold_{i:05}' : f'contig_{i:05}'
             for i in range(4000)
         },
         h5opts
